@@ -3,7 +3,7 @@
 Various functions for processing EnergyPlus models and retrieving results in
 different forms.
 """
-
+import io
 import itertools
 import logging as lg
 import math
@@ -19,18 +19,23 @@ from collections import defaultdict
 from io import IOBase, StringIO
 from itertools import chain
 from math import isclose
-from typing import Any, Optional, Union
+from typing import IO, Iterable, Optional, Union
 
 import eppy
 import pandas as pd
 from energy_pandas import EnergySeries
 from eppy.bunch_subclass import BadEPFieldError
-from eppy.easyopen import getiddfile
 from eppy.EPlusInterfaceFunctions.eplusdata import Eplusdata
+from eppy.idf_msequence import Idf_MSequence
 from eppy.modeleditor import IDDNotSetError, namebunch, newrawobject
-from geomeppy import IDF as geomIDF
+from geomeppy import IDF as GeomIDF
 from geomeppy.geom.polygons import Polygon3D
 from geomeppy.patches import EpBunch, idfreader1, obj2bunch
+from geomeppy.recipes import (
+    _has_correct_orientation,
+    _is_window,
+    window_vertices_given_wall,
+)
 from pandas import DataFrame, Series
 from pandas.errors import ParserError
 from path import Path
@@ -47,7 +52,7 @@ from archetypal.eplus_interface.exceptions import (
 from archetypal.eplus_interface.expand_objects import ExpandObjectsThread
 from archetypal.eplus_interface.slab import SlabThread
 from archetypal.eplus_interface.transition import TransitionThread
-from archetypal.eplus_interface.version import EnergyPlusVersion, get_eplus_dirs
+from archetypal.eplus_interface.version import EnergyPlusVersion
 from archetypal.idfclass.meters import Meters
 from archetypal.idfclass.outputs import Outputs
 from archetypal.idfclass.reports import get_report
@@ -69,7 +74,7 @@ def find_and_launch(app_name, app_path_guess, file_path):
     )
 
 
-class IDF(geomIDF):
+class IDF(GeomIDF):
     """Class for loading and parsing idf models.
 
     This is the starting point to run simulations and retrieving results.
@@ -86,7 +91,6 @@ class IDF(geomIDF):
 
     # dependencies: dict of <dependant value: independent value>
     _dependencies = {
-        "iddname": ["idfname", "as_version"],
         "idd_info": ["iddname", "idfname"],
         "idd_index": ["iddname", "idfname"],
         "idd_version": ["iddname", "idfname"],
@@ -175,9 +179,9 @@ class IDF(geomIDF):
 
     def __init__(
         self,
-        idfname=None,
+        idfname: Optional[Union[str, IO, Path]] = None,
         epw=None,
-        as_version=settings.ep_version,
+        as_version: Union[str, EnergyPlusVersion] = settings.ep_version,
         annual=False,
         design_day=False,
         expandobjects=False,
@@ -190,11 +194,12 @@ class IDF(geomIDF):
         output_suffix="L",
         epmacro=False,
         keep_data=True,
-        keep_data_err=False,
+        keep_data_err=True,
         position=0,
         name=None,
         output_directory=None,
         outputtype="standard",
+        iddname: Optional[Union[str, IO, Path]] = None,
         **kwargs,
     ):
         """Initialize an IDF object.
@@ -230,6 +235,7 @@ class IDF(geomIDF):
             include = []
         self.idfname = idfname
         self.epw = epw
+        self.file_version = kwargs.get("file_version", None)
         self.as_version = as_version if as_version else settings.ep_version
         self._custom_processes = custom_processes
         self._include = include
@@ -246,12 +252,17 @@ class IDF(geomIDF):
         self.prep_outputs = prep_outputs
         self._position = position
         self.output_prefix = None
-        self.name = self.idfname.basename() if isinstance(self.idfname, Path) else name
+        self.name = (
+            name
+            if name is not None
+            else self.idfname.basename()
+            if isinstance(self.idfname, Path)
+            else None
+        )
         self.output_directory = output_directory
 
         # Set dependants to None
-        self.file_version = kwargs.get("file_version", None)
-        self._iddname = None
+        self._iddname = Path(iddname) if iddname is not None else None
         self._idd_info = None
         self._idd_index = None
         self._idd_version = None
@@ -289,23 +300,24 @@ class IDF(geomIDF):
                 self.upgrade(to_version=self.as_version, overwrite=False)
         finally:
             # Set model outputs
-            self._outputs = Outputs(idf=self)
+            self._outputs = Outputs(idf=self, include_html=False, include_sqlite=False)
             if self.prep_outputs:
-                (
-                    self._outputs.add_basics()
-                    .add_umi_template_outputs()
-                    .add_custom(outputs=self.prep_outputs)
-                    .add_profile_gas_elect_ouputs()
-                    .apply()
-                )
+                self._outputs.include_html = True
+                self._outputs.include_sqlite = True
+                self._outputs.add_basics()
+                if isinstance(self.prep_outputs, list):
+                    self._outputs.add_custom(outputs=self.prep_outputs)
+                self._outputs.add_profile_gas_elect_outputs()
+                self._outputs.add_umi_template_outputs()
+                self._outputs.apply()
 
     @property
     def outputtype(self):
+        """Get or set the outputtype for the idf string representation of self."""
         return self._outputtype
 
     @outputtype.setter
     def outputtype(self, value):
-        """Get or set the outputtype for the idf string representation of self."""
         assert value in self.OUTPUTTYPES, (
             f'Invalid input "{value}" for output_type.'
             f"\nOutput type must be one of the following: {self.OUTPUTTYPES}"
@@ -328,17 +340,52 @@ class IDF(geomIDF):
         body += sim_info
         return f"<{body}>"
 
+    def __copy__(self):
+        """Get a copy of self."""
+        return self.copy()
+
     @classmethod
-    def from_example_files(cls, example_name, **kwargs):
-        """Load an IDF model from the ExampleFiles folder by name."""
+    def from_example_files(cls, example_name, epw=None, **kwargs):
+        """Load an IDF model from the ExampleFiles folder by name.
+
+        Examples:
+            idf = IDF.from_example_files(
+                "minimal",
+                "USA_IL_Chicago-OHare.Intl.AP.725300_TMY3.epw"
+            )
+
+        Args:
+            example_name (str): The name of the example file to load.
+            epw (str): The name of the weather file contained the WeatherData folder
+                or the path to a specific weather file.
+            **kwargs: keyword arguments passed to the IDF constructor.
+
+        Returns:
+            (IDF): An IDF model.
+        """
+        from pathlib import Path as Pathlib
+
+        example_name = Path(example_name)
         file = next(
             iter(
-                (get_eplus_dirs(settings.ep_version) / "ExampleFiles").files(
-                    example_name
-                )
+                Pathlib(
+                    EnergyPlusVersion.current().current_install_dir / "ExampleFiles"
+                ).rglob(f"{example_name.stem}.idf")
             )
         )
-        return cls(file, **kwargs)
+        if epw is not None:
+            epw = Path(epw)
+            if not epw.exists():
+                epw = next(
+                    iter(
+                        Pathlib(
+                            EnergyPlusVersion.current().current_install_dir
+                            / "WeatherData"
+                        ).rglob(f"{epw.stem}.epw")
+                    ),
+                    epw,
+                )
+        return cls(file, epw=epw, **kwargs)
 
     def setiddname(self, iddname, testing=False):
         """Set EnergyPlus IDD path for model.
@@ -384,6 +431,7 @@ class IDF(geomIDF):
         """Set the IDD to be used by eppy.
 
         Args:
+            iddindex:
             iddinfo (list): Comments and metadata about fields in the IDD.
             block (list): Field names in the IDD.
         """
@@ -451,23 +499,22 @@ class IDF(geomIDF):
 
     @property
     def iddname(self) -> Path:
-        """Return the iddname used to parse the idf model."""
+        """Get or set the iddname path used to parse the idf model."""
         if self._iddname is None:
             if self.file_version > self.as_version:
                 raise EnergyPlusVersionError(
                     f"{self.as_version} cannot be lower then "
                     f"the version number set in the file: {self.file_version}"
                 )
-            idd_filename = Path(getiddfile(str(self.file_version))).expand()
-            if not idd_filename.exists():
-                # Try finding the one in IDFVersionsUpdater
-                idd_filename = (
-                    self.idfversionupdater_dir / f"V"
-                    f"{self.file_version.dash}-Energy+.idd"
-                ).expand()
-            self._iddname = idd_filename
+            self._iddname = self.file_version.current_idd_path
         return self._iddname
 
+    @iddname.setter
+    def iddname(self, value):
+        if value is not None:
+            value = Path(value)
+        self._iddname = value
+
     @property
     def file_version(self) -> EnergyPlusVersion:
         """Return the :class:`EnergyPlusVersion` of the idf text file."""
@@ -664,6 +711,10 @@ class IDF(geomIDF):
 
     @prep_outputs.setter
     def prep_outputs(self, value):
+        assert isinstance(value, (bool, list)), (
+            f"Expected bool or list of dict for "
+            f"SimulationOutput outputs. Got {type(value)}."
+        )
         self._prep_outputs = value
 
     @property
@@ -683,8 +734,9 @@ class IDF(geomIDF):
         """Path: The output directory based on the hashing of the original file.
 
         Notes:
-            The hashing is performed before transitions or modifications. The directory is not created! Use
-            `self.output_directory.makedir_p()` to create it without an error if it exists.
+            The hashing is performed before transitions or modifications. The directory
+            is not created! Use `self.output_directory.makedir_p()` to create it
+            without an error if it exists.
         """
         if self._output_directory is None:
             cache_filename = self._original_cache
@@ -766,7 +818,9 @@ class IDF(geomIDF):
         Uses the current module's ep_version.
         """
         return (
-            get_eplus_dirs(settings.ep_version) / "PreProcess" / "IDFVersionUpdater"
+            EnergyPlusVersion.current().current_install_dir
+            / "PreProcess"
+            / "IDFVersionUpdater"
         ).expand()
 
     @property
@@ -848,10 +902,8 @@ class IDF(geomIDF):
         else:
             filepath = self.idfname
 
-        import subprocess
-
         app_path_guess = (
-            get_eplus_dirs(self.file_version.dash) / "PreProcess" / "IDFEditor"
+            self.file_version.current_install_dir / "PreProcess" / "IDFEditor"
         )
         find_and_launch("IDFEditor", app_path_guess, filepath.abspath())
 
@@ -859,10 +911,17 @@ class IDF(geomIDF):
         """Open last simulation in Ep-Launch."""
         filepath, *_ = self.simulation_dir.files("*.idf")
 
-        import subprocess
-        app_path_guess = get_eplus_dirs(self.file_version.dash)
+        app_path_guess = self.file_version.current_install_dir
         find_and_launch("EP-Launch", app_path_guess, filepath.abspath())
 
+    def open_err(self):
+        """Open last simulation err file in texteditor."""
+        import webbrowser
+
+        filepath, *_ = self.simulation_dir.files("*.err")
+
+        webbrowser.open(filepath.abspath())
+
     def open_mdd(self):
         """Open .mdd file in browser.
 
@@ -1215,7 +1274,7 @@ class IDF(geomIDF):
                 self._variables = Variables(self)
         return self._variables
 
-    def simulate(self, **kwargs):
+    def simulate(self, force=False, **kwargs):
         """Execute EnergyPlus.
 
         Specified kwargs overwrite IDF parameters. ExpandObjects, Basement and Slab
@@ -1223,6 +1282,10 @@ class IDF(geomIDF):
 
         Does not return anything.
 
+        Args:
+            force (bool): Force simulation even though results exist in
+                `self.simulation_dir`.
+
         Keyword Args:
             eplus_file (str): path to the idf file.
             weather_file (str): path to the EPW weather file.
@@ -1277,9 +1340,14 @@ class IDF(geomIDF):
                 energyplus command.
 
         See Also:
-            :meth:`simulation_files`, :meth:`processed_results` for simulation outputs.
+            :attr:`IDF.simulation_files`, :attr:`IDF.processed_results` for simulation
+            outputs.
 
         """
+        if (
+            self.simulation_dir.exists() and not force
+        ):  # don't simulate if results exists
+            return self
         # First, update keys with new values
         for key, value in kwargs.items():
             if f"_{key}" in self.__dict__.keys():
@@ -1307,57 +1375,83 @@ class IDF(geomIDF):
         # Todo: Add EpMacro Thread -> if exist in.imf "%program_path%EPMacro"
         # Run the expandobjects program if necessary
         tmp = (
-            self.output_directory.makedirs_p() / "expandobjects_run_" + str(uuid.uuid1())[0:8]
+            self.output_directory.makedirs_p() / "expandobjects_run_"
+            + str(uuid.uuid1())[0:8]
         ).mkdir()
         # Run the ExpandObjects preprocessor program
         expandobjects_thread = ExpandObjectsThread(self, tmp)
-        expandobjects_thread.start()
-        expandobjects_thread.join()
-        while expandobjects_thread.is_alive():
-            time.sleep(1)
-        tmp.rmtree(ignore_errors=True)
-        e = expandobjects_thread.exception
-        if e is not None:
-            raise e
+        try:
+            expandobjects_thread.start()
+            expandobjects_thread.join()
+            # Give time to the subprocess to finish completely
+            while expandobjects_thread.is_alive():
+                time.sleep(1)
+        except (KeyboardInterrupt, SystemExit):
+            expandobjects_thread.stop()
+        finally:
+            tmp.rmtree(ignore_errors=True)
+            e = expandobjects_thread.exception
+            if e is not None:
+                raise e
 
         # Run the Basement preprocessor program if necessary
         tmp = (
-            self.output_directory.makedirs_p() / "runBasement_run_" + str(uuid.uuid1())[0:8]
+            self.output_directory.makedirs_p() / "runBasement_run_"
+            + str(uuid.uuid1())[0:8]
         ).mkdir()
         basement_thread = BasementThread(self, tmp)
-        basement_thread.start()
-        basement_thread.join()
-        while basement_thread.is_alive():
-            time.sleep(1)
-        tmp.rmtree(ignore_errors=True)
-        e = basement_thread.exception
-        if e is not None:
-            raise e
+        try:
+            basement_thread.start()
+            basement_thread.join()
+            # Give time to the subprocess to finish completely
+            while basement_thread.is_alive():
+                time.sleep(1)
+        except KeyboardInterrupt:
+            basement_thread.stop()
+        finally:
+            tmp.rmtree(ignore_errors=True)
+            e = basement_thread.exception
+            if e is not None:
+                raise e
 
         # Run the Slab preprocessor program if necessary
-        tmp = (self.output_directory.makedirs_p() / "runSlab_run_" + str(uuid.uuid1())[0:8]).mkdir()
+        tmp = (
+            self.output_directory.makedirs_p() / "runSlab_run_" + str(uuid.uuid1())[0:8]
+        ).mkdir()
         slab_thread = SlabThread(self, tmp)
-        slab_thread.start()
-        slab_thread.join()
-        while slab_thread.is_alive():
-            time.sleep(1)
-        tmp.rmtree(ignore_errors=True)
-        e = slab_thread.exception
-        if e is not None:
-            raise e
+        try:
+            slab_thread.start()
+            slab_thread.join()
+            # Give time to the subprocess to finish completely
+            while slab_thread.is_alive():
+                time.sleep(1)
+        except KeyboardInterrupt:
+            slab_thread.stop()
+        finally:
+            tmp.rmtree(ignore_errors=True)
+            e = slab_thread.exception
+            if e is not None:
+                raise e
 
         # Run the energyplus program
-        tmp = (self.output_directory.makedirs_p() / "eplus_run_" + str(uuid.uuid1())[0:8]).mkdir()
+        tmp = (
+            self.output_directory.makedirs_p() / "eplus_run_" + str(uuid.uuid1())[0:8]
+        ).mkdir()
         running_simulation_thread = EnergyPlusThread(self, tmp)
-        running_simulation_thread.start()
-        running_simulation_thread.join()
-        while running_simulation_thread.is_alive():
-            time.sleep(1)
-        tmp.rmtree(ignore_errors=True)
-        e = running_simulation_thread.exception
-        if e is not None:
-            raise e
-        return self
+        try:
+            running_simulation_thread.start()
+            running_simulation_thread.join()
+            # Give time to the subprocess to finish completely
+            while running_simulation_thread.is_alive():
+                time.sleep(1)
+        except KeyboardInterrupt:
+            running_simulation_thread.stop()
+        finally:
+            tmp.rmtree(ignore_errors=True)
+            e = running_simulation_thread.exception
+            if e is not None:
+                raise e
+            return self
 
     def savecopy(self, filename, lineendings="default", encoding="latin-1"):
         """Save a copy of the file with the filename passed.
@@ -1376,6 +1470,15 @@ class IDF(geomIDF):
         super(IDF, self).save(filename, lineendings, encoding)
         return Path(filename)
 
+    def copy(self):
+        """Return a copy of self as an in memory IDF.
+
+        The copy is a new IDF object with the same parameters and arguments as self
+        but is not attached to an file. Use IDF.saveas("idfname.idf", inplace=True)
+        to save the copy to a file inplace. self.idfname will now be idfname.idf
+        """
+        return self.saveas(io.StringIO(""))
+
     def save(self, lineendings="default", encoding="latin-1", **kwargs):
         """Write the IDF model to the text file.
 
@@ -1383,8 +1486,6 @@ class IDF(geomIDF):
         simulation results.
 
         Args:
-            filename (str): Filepath to save the file. If None then use the IDF.idfname
-                parameter. Also accepts a file handle.
             lineendings (str) : Line endings to use in the saved file. Options are
                 'default', 'windows' and 'unix' the default is 'default' which uses
                 the line endings for the current system.
@@ -1399,7 +1500,9 @@ class IDF(geomIDF):
         log(f"saved '{self.name}' at '{self.idfname}'")
         return self
 
-    def saveas(self, filename, lineendings="default", encoding="latin-1"):
+    def saveas(
+        self, filename, lineendings="default", encoding="latin-1", inplace=False
+    ):
         """Save the IDF model as.
 
         Writes a new text file and load a new instance of the IDF class (new object).
@@ -1412,6 +1515,8 @@ class IDF(geomIDF):
                 the line endings for the current system.
             encoding (str): Encoding to use for the saved file. The default is
                 'latin-1' which is compatible with the EnergyPlus IDFEditor.
+            inplace (bool): If True, applies the new filename to self directly,
+                else a new object is returned with the new filename.
 
         Returns:
             IDF: A new IDF object based on the new location file.
@@ -1441,8 +1546,18 @@ class IDF(geomIDF):
                     name = Path(name).basename()
                 else:
                     name = file.basename()
-                file.copy(as_idf.simulation_dir / name)
-        return as_idf
+                try:
+                    file.copy(as_idf.simulation_dir / name)
+                except shutil.SameFileError:
+                    # A copy of self would have the same files in the simdir and
+                    # throw an error.
+                    pass
+        if inplace:
+            # If inplace, replace content of self with content of as_idf.
+            self.__dict__.update(as_idf.__dict__)
+        else:
+            # return the new object.
+            return as_idf
 
     def process_results(self):
         """Return the list of processed results.
@@ -1485,6 +1600,31 @@ class IDF(geomIDF):
         else:
             return results
 
+    def add_idf_object_from_idf_string(self, idf_string):
+        """Add an IDF object (or more than one) from an EnergyPlus text string.
+
+        Args:
+            idf_string (str): A text string fully describing an EnergyPlus object.
+        """
+        # Load the idf_string as a new model.
+        loaded_string = IDF(
+            StringIO(idf_string),
+            file_version=self.file_version,
+            as_version=self.as_version,
+            prep_outputs=False,
+        )
+
+        # Loop on all objects and using self.newidfobject
+        added_objects = []
+        for sequence in loaded_string.idfobjects.values():
+            if sequence:
+                for obj in sequence:
+                    data = obj.to_dict()
+                    key = data.pop("key")
+                    added_objects.append(self.newidfobject(key=key.upper(), **data))
+        del loaded_string  # remove loaded_string model
+        return added_objects
+
     def upgrade(self, to_version=None, overwrite=True):
         """`EnergyPlus` idf version updater using local transition program.
 
@@ -1528,13 +1668,13 @@ class IDF(geomIDF):
             tmp = (
                 self.output_directory / "Transition_run_" + str(uuid.uuid1())[0:8]
             ).makedirs_p()
-            slab_thread = TransitionThread(self, tmp, overwrite=overwrite)
-            slab_thread.start()
-            slab_thread.join()
-            while slab_thread.is_alive():
+            transition_thread = TransitionThread(self, tmp, overwrite=overwrite)
+            transition_thread.start()
+            transition_thread.join()
+            while transition_thread.is_alive():
                 time.sleep(1)
             tmp.rmtree(ignore_errors=True)
-            e = slab_thread.exception
+            e = transition_thread.exception
             if e is not None:
                 raise e
 
@@ -1801,7 +1941,7 @@ class IDF(geomIDF):
         except BadEPFieldError as e:
             raise e
         else:
-            # If object is supposed to be 'unique-object', deletes all objects to be
+            # If object is supposed to be 'unique-object', delete all objects to be
             # sure there is only one of them when creating new object
             # (see following line)
             if "unique-object" in set().union(
@@ -1809,22 +1949,23 @@ class IDF(geomIDF):
             ):
                 for obj in existing_objs:
                     self.removeidfobject(obj)
-                    self.addidfobject(new_object)
                     log(
                         f"{obj} is a 'unique-object'; Removed and replaced with"
                         f" {new_object}",
                         lg.DEBUG,
                     )
+                self.addidfobject(new_object)
                 return new_object
             if new_object in existing_objs:
-                # If obj already exists, simply return
+                # If obj already exists, simply return the existing one.
                 log(
                     f"object '{new_object}' already exists in {self.name}. "
                     f"Skipping.",
                     lg.DEBUG,
                 )
-                return new_object
+                return next(x for x in existing_objs if x == new_object)
             elif new_object not in existing_objs and new_object.nameexists():
+                # Object does not exist (because not equal) but Name exists.
                 obj = self.getobject(
                     key=new_object.key.upper(), name=new_object.Name.upper()
                 )
@@ -1856,6 +1997,17 @@ class IDF(geomIDF):
         self._reset_dependant_vars("idfobjects")
         return new_object
 
+    def addidfobjects(self, new_objects):
+        """Add multiple IDF objects to the model.
+
+        Resetting dependent variables will wait after all objects have been added.
+        """
+        for new_object in new_objects:
+            key = new_object.key.upper()
+            self.idfobjects[key].append(new_object)
+        self._reset_dependant_vars("idfobjects")
+        return new_objects
+
     def removeidfobject(self, idfobject):
         """Remove an IDF object from the model.
 
@@ -1866,8 +2018,20 @@ class IDF(geomIDF):
         self.idfobjects[key].remove(idfobject)
         self._reset_dependant_vars("idfobjects")
 
-    def anidfobject(self, key, aname="", **kwargs) -> EpBunch:
-        # type: (str, str, **Any) -> EpBunch
+    def removeidfobjects(self, idfobjects: Iterable[EpBunch]):
+        """Remove an IDF object from the model.
+
+        Resetting dependent variables will wait after all objects have been removed.
+
+        Args:
+            idfobjects: The object to remove from the model.
+        """
+        for idfobject in idfobjects:
+            key = idfobject.key.upper()
+            self.idfobjects[key].remove(idfobject)
+        self._reset_dependant_vars("idfobjects")
+
+    def anidfobject(self, key: str, aname: str = "", **kwargs) -> EpBunch:
         """Define and create an object, but don't add it to the model.
 
         See :func:`~archetypal.idfclass.idf.IDF.newidfobject`). If you don't specify
@@ -2006,9 +2170,9 @@ class IDF(geomIDF):
         used_schedules = []
         all_schedules = self._get_all_schedules(yearly_only=yearly_only)
         for object_name in self.idfobjects:
-            for object in self.idfobjects[object_name]:
-                if object.key.upper() not in schedule_types:
-                    for fieldvalue in object.fieldvalues:
+            for obj in self.idfobjects[object_name]:
+                if obj.key.upper() not in schedule_types:
+                    for fieldvalue in obj.fieldvalues:
                         try:
                             if (
                                 fieldvalue.upper() in all_schedules.keys()
@@ -2019,6 +2183,30 @@ class IDF(geomIDF):
                             pass
         return used_schedules
 
+    @property
+    def width(self):
+        """Get the width of the building [m]."""
+        bbox = self.bounding_box()
+        return max(bbox.xs) - min(bbox.xs)
+
+    @property
+    def length(self):
+        """Get the length of the building [m]."""
+        bbox = self.bounding_box()
+        return max(bbox.ys) - min(bbox.ys)
+
+    def resize(self, width, length):
+        """Resize the floor plate to x and y [meters].
+
+        Args:
+            width (float): The new width [m] of the building (x axis).
+            length (float): The new length [m] of the building (y axis).
+        """
+        x_scale = width / self.width
+        y_scale = length / self.length
+        self.scale(x_scale, axes="x")
+        self.scale(y_scale, axes="y")
+
     def rename(self, objkey, objname, newname):
         """Rename all the references to this objname.
 
@@ -2053,6 +2241,104 @@ class IDF(geomIDF):
         theobject[fieldname] = newname
         return theobject
 
+    def set_wwr(
+        self,
+        wwr: float = None,
+        construction: Optional[str] = None,
+        force: bool = False,
+        wwr_map: Optional[dict] = None,
+        orientation: Optional[str] = None,
+        surfaces: Optional[Iterable] = None,
+    ):
+        """Set Window-to-Wall Ratio of external walls.
+
+        Different WWR can be applied to specific wall orientations using the
+        `wwr_map` dictionary. This map is a dict of wwr values, keyed by
+        `wall.azimuth`, which overrides the default passed as `wwr`. Note that
+        `wall.azimuth` is rounded to the closest integer.
+
+        They can also be applied to walls oriented to a compass point, e.g. north,
+        which will apply to walls which have an azimuth within 45 degrees of due north.
+
+        Args:
+            wwr: The window to wall ratio to apply to all orientations. If wwr_map is
+                specified, `wwr` is the default wwr for oreintations not defined in
+                the mapping.
+            construction: Name of a window construction to apply.
+            force: True to create windows on walls that don't have any.
+            wwr_map: Mapping from wall orientation (azimuth) to WWR, e.g.
+                {180: 0.25, 90: 0.2}.
+            orientation: One of "north", "east", "south", "west". Walls within 45
+                degrees will be affected.
+            surfaces: Iterable of surfaces to set the window to wall ratio of.
+        """
+        # Taken from `geomeppy.idf.IDF.set_wwr` since pull request is not being
+        # reviewed as of 2021-11-10.
+
+        try:
+            ggr: Optional[Idf_MSequence] = self.idfobjects["GLOBALGEOMETRYRULES"][0]
+        except IndexError:
+            ggr = None
+
+            # check orientation
+        orientations = {
+            "north": 0.0,
+            "east": 90.0,
+            "south": 180.0,
+            "west": 270.0,
+            None: None,
+        }
+        degrees = orientations.get(orientation, None)
+        external_walls = filter(
+            lambda x: x.Outside_Boundary_Condition.lower() == "outdoors",
+            surfaces or self.getsurfaces("wall"),
+        )
+        external_walls = filter(
+            lambda x: _has_correct_orientation(x, degrees), external_walls
+        )
+        subsurfaces = self.getsubsurfaces()
+        base_wwr = wwr
+        for wall in external_walls:
+            # get any subsurfaces on the wall
+            wall_subsurfaces = list(
+                filter(lambda x: x.Building_Surface_Name == wall.Name, subsurfaces)
+            )
+
+            if wall_subsurfaces and not construction:
+                constructions = list(
+                    {
+                        wss.Construction_Name
+                        for wss in wall_subsurfaces
+                        if _is_window(wss)
+                    }
+                )
+                if len(constructions) > 1:
+                    raise ValueError(
+                        'Not all subsurfaces on wall "{name}" have the same construction'.format(
+                            name=wall.Name
+                        )
+                    )
+                construction = constructions[0]
+            wwr = (wwr_map or {}).get(round(wall.azimuth), base_wwr)
+            if not wwr:
+                continue
+            if len(wall_subsurfaces) == 0 and not force:
+                # Don't create windows on walls that don't have any windows already.
+                continue
+            # remove all subsurfaces
+            for ss in wall_subsurfaces:
+                self.removeidfobject(ss)
+            coords = window_vertices_given_wall(wall, wwr)
+            window = self.newidfobject(
+                "FENESTRATIONSURFACE:DETAILED",
+                Name="%s window" % wall.Name,
+                Surface_Type="Window",
+                Construction_Name=construction or "",
+                Building_Surface_Name=wall.Name,
+                View_Factor_to_Ground="autocalculate",  # from the surface angle
+            )
+            window.setcoords(coords, ggr)
+
     def _energy_series(
         self,
         energy_out_variable_name,
@@ -2082,9 +2368,9 @@ class IDF(geomIDF):
     def _execute_transitions(self, idf_file, to_version, **kwargs):
         trans_exec = {
             EnergyPlusVersion(
-                re.search(r"to-V(([\d])-([\d])-([\d]))", exec).group(1)
-            ): exec
-            for exec in self.idfversionupdater_dir.files("Transition-V*")
+                re.search(r"to-V(([\d])-([\d])-([\d]))", executable).group(1)
+            ): executable
+            for executable in self.idfversionupdater_dir.files("Transition-V*")
         }
 
         transitions = [
